@@ -1,4 +1,4 @@
-/**
+/*
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -106,6 +106,16 @@ public interface KeyValueStorage extends Closeable {
      */
     default void compact(byte[] firstKey, byte[] lastKey) throws IOException {}
 
+    /**
+     * Compact storage full range.
+     */
+    default void compact() throws IOException {}
+
+    /**
+     * Get storage path.
+     */
+    String getDBPath();
+
     /**
      * Get an iterator over to scan sequentially through all the keys in the
      * database.
@@ -168,9 +178,5 @@ public interface KeyValueStorage extends Closeable {
         void clear();
 
         void flush() throws IOException;
-
-        default int batchCount() {
-            return -1;
-        }
     }
 }
