@@ -21,20 +21,11 @@ package org.apache.bookkeeper.proto.checksum;
 import com.scurrilous.circe.checksum.Crc32cIntChecksum;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufAllocator;
-import io.netty.util.concurrent.FastThreadLocal;
 import lombok.extern.slf4j.Slf4j;
-import org.apache.commons.lang3.mutable.MutableInt;
 
 @Slf4j
 class CRC32CDigestManager extends DigestManager {
 
-    private static final FastThreadLocal<MutableInt> currentCrc = new FastThreadLocal<MutableInt>() {
-        @Override
-        protected MutableInt initialValue() throws Exception {
-            return new MutableInt(0);
-        }
-    };
-
     public CRC32CDigestManager(long ledgerId, boolean useV2Protocol, ByteBufAllocator allocator) {
         super(ledgerId, useV2Protocol, allocator);
     }
@@ -45,16 +36,17 @@ class CRC32CDigestManager extends DigestManager {
     }
 
     @Override
-    void populateValueAndReset(ByteBuf buf) {
-        MutableInt current = currentCrc.get();
-        buf.writeInt(current.intValue());
-        current.setValue(0);
+    boolean isInt32Digest() {
+        return true;
     }
 
     @Override
-    void update(ByteBuf data) {
-        MutableInt current = currentCrc.get();
-        final int lastCrc = current.intValue();
-        current.setValue(Crc32cIntChecksum.resumeChecksum(lastCrc, data));
+    void populateValueAndReset(int digest, ByteBuf buf) {
+        buf.writeInt(digest);
+    }
+
+    @Override
+    int update(int digest, ByteBuf data, int offset, int len) {
+        return Crc32cIntChecksum.resumeChecksum(digest, data, offset, len);
     }
 }
