@@ -22,8 +22,8 @@ package org.apache.bookkeeper.bookie;
 import com.google.common.util.concurrent.RateLimiter;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufAllocator;
-
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.EnumSet;
 import java.util.List;
@@ -31,7 +31,6 @@ import java.util.Optional;
 import java.util.PrimitiveIterator;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.atomic.AtomicBoolean;
-
 import org.apache.bookkeeper.bookie.CheckpointSource.Checkpoint;
 import org.apache.bookkeeper.common.util.Watcher;
 import org.apache.bookkeeper.conf.ServerConfiguration;
@@ -41,7 +40,7 @@ import org.apache.bookkeeper.stats.StatsLogger;
 /**
  * A mock for running tests that require ledger storage.
  */
-public class MockLedgerStorage implements LedgerStorage {
+public class MockLedgerStorage implements CompactableLedgerStorage {
 
     private static class LedgerInfo {
         boolean limbo = false;
@@ -58,6 +57,7 @@ public class MockLedgerStorage implements LedgerStorage {
 
     private final ConcurrentHashMap<Long, LedgerInfo> ledgers = new ConcurrentHashMap<>();
     private final EnumSet<StorageState> storageStateFlags = EnumSet.noneOf(StorageState.class);
+    private final List<EntryLocation> entryLocations = new ArrayList<>();
 
     @Override
     public void initialize(ServerConfiguration conf,
@@ -242,7 +242,7 @@ public class MockLedgerStorage implements LedgerStorage {
 
     @Override
     public void deleteLedger(long ledgerId) throws IOException {
-        throw new UnsupportedOperationException("Not supported in mock, implement if you need it");
+        ledgers.remove(ledgerId);
     }
 
     @Override
@@ -262,32 +262,56 @@ public class MockLedgerStorage implements LedgerStorage {
 
     @Override
     public LedgerStorage getUnderlyingLedgerStorage() {
-        return LedgerStorage.super.getUnderlyingLedgerStorage();
+        return CompactableLedgerStorage.super.getUnderlyingLedgerStorage();
     }
 
     @Override
     public void forceGC() {
-        LedgerStorage.super.forceGC();
+        CompactableLedgerStorage.super.forceGC();
     }
 
     @Override
-    public void forceGC(boolean forceMajor, boolean forceMinor) {
-        LedgerStorage.super.forceGC(forceMajor, forceMinor);
+    public void forceGC(Boolean forceMajor, Boolean forceMinor) {
+        CompactableLedgerStorage.super.forceGC(forceMajor, forceMinor);
+    }
+
+    public void suspendMinorGC() {
+        CompactableLedgerStorage.super.suspendMinorGC();
+    }
+
+    public void suspendMajorGC() {
+        CompactableLedgerStorage.super.suspendMajorGC();
+    }
+
+    public void resumeMinorGC() {
+        CompactableLedgerStorage.super.resumeMinorGC();
+    }
+
+    public void resumeMajorGC() {
+        CompactableLedgerStorage.super.suspendMajorGC();
+    }
+
+    public boolean isMajorGcSuspended() {
+        return CompactableLedgerStorage.super.isMajorGcSuspended();
+    }
+
+    public boolean isMinorGcSuspended() {
+        return CompactableLedgerStorage.super.isMinorGcSuspended();
     }
 
     @Override
     public List<DetectedInconsistency> localConsistencyCheck(Optional<RateLimiter> rateLimiter) throws IOException {
-        return LedgerStorage.super.localConsistencyCheck(rateLimiter);
+        return CompactableLedgerStorage.super.localConsistencyCheck(rateLimiter);
     }
 
     @Override
     public boolean isInForceGC() {
-        return LedgerStorage.super.isInForceGC();
+        return CompactableLedgerStorage.super.isInForceGC();
     }
 
     @Override
     public List<GarbageCollectionStatus> getGarbageCollectionStatus() {
-        return LedgerStorage.super.getGarbageCollectionStatus();
+        return CompactableLedgerStorage.super.getGarbageCollectionStatus();
     }
 
     @Override
@@ -295,6 +319,25 @@ public class MockLedgerStorage implements LedgerStorage {
         throw new UnsupportedOperationException("Not supported in mock, implement if you need it");
     }
 
+    @Override
+    public Iterable<Long> getActiveLedgersInRange(long firstLedgerId, long lastLedgerId)
+            throws IOException {
+        throw new UnsupportedOperationException("Not supported in mock, implement if you need it");
+    }
+
+    public List<EntryLocation> getUpdatedLocations() {
+        return entryLocations;
+    }
+
+    @Override
+    public void updateEntriesLocations(Iterable<EntryLocation> locations) throws IOException {
+        synchronized (entryLocations) {
+            for (EntryLocation l : locations) {
+                entryLocations.add(l);
+            }
+        }
+    }
+
     @Override
     public EnumSet<StorageState> getStorageStateFlags() throws IOException {
         return storageStateFlags;
@@ -309,4 +352,7 @@ public class MockLedgerStorage implements LedgerStorage {
     public void clearStorageStateFlag(StorageState flag) throws IOException {
         storageStateFlags.remove(flag);
     }
+
+    @Override
+    public void flushEntriesLocationsIndex() throws IOException { }
 }
