@@ -27,22 +27,18 @@ import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.verify;
 
 import com.google.common.collect.Lists;
-
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
-
 import org.apache.bookkeeper.client.api.LedgerMetadata;
 import org.apache.bookkeeper.client.api.WriteFlag;
 import org.apache.bookkeeper.common.concurrent.FutureUtils;
 import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.net.BookieSocketAddress;
 import org.apache.bookkeeper.versioning.Versioned;
-
 import org.junit.Assert;
 import org.junit.Test;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -58,7 +54,7 @@ public class HandleFailuresTest {
     private static final BookieId b4 = new BookieSocketAddress("b4", 3181).toBookieId();
     private static final BookieId b5 = new BookieSocketAddress("b5", 3181).toBookieId();
 
-    @Test
+    @Test(timeout = 30000)
     public void testChangeTriggeredOneTimeForOneFailure() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -81,7 +77,7 @@ public class HandleFailuresTest {
         Assert.assertEquals(lh.getLedgerMetadata().getAllEnsembles().get(0L), Lists.newArrayList(b4, b2, b3));
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testSecondFailureOccursWhileFirstBeingHandled() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -129,7 +125,7 @@ public class HandleFailuresTest {
         Assert.assertTrue(lh.getLedgerMetadata().getAllEnsembles().get(0L).contains(b5));
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testHandlingFailuresOneBookieFailsImmediately() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -149,7 +145,7 @@ public class HandleFailuresTest {
         Assert.assertEquals(lh.getLedgerMetadata().getAllEnsembles().get(0L), Lists.newArrayList(b4, b2, b3));
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testHandlingFailuresOneBookieFailsAfterOneEntry() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -172,7 +168,7 @@ public class HandleFailuresTest {
         Assert.assertEquals(lh.getLedgerMetadata().getLastEntryId(), 1L);
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testHandlingFailuresMultipleBookieFailImmediatelyNotEnoughToReplace() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -198,7 +194,7 @@ public class HandleFailuresTest {
         }
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testHandlingFailuresMultipleBookieFailAfterOneEntryNotEnoughToReplace() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -226,7 +222,7 @@ public class HandleFailuresTest {
         }
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testClientClosesWhileFailureHandlerInProgress() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -268,7 +264,7 @@ public class HandleFailuresTest {
         Assert.assertEquals(lh.getLedgerMetadata().getLastEntryId(), LedgerHandle.INVALID_ENTRY_ID);
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testMetadataSetToClosedDuringFailureHandler() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -312,7 +308,7 @@ public class HandleFailuresTest {
         Assert.assertEquals(lh.getLedgerMetadata().getLastEntryId(), 1234L);
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testMetadataSetToInRecoveryDuringFailureHandler() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -354,7 +350,7 @@ public class HandleFailuresTest {
         Assert.assertEquals(lh.getLedgerMetadata().getAllEnsembles().get(0L), Lists.newArrayList(b1, b2, b3));
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testOldEnsembleChangedDuringFailureHandler() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -407,7 +403,7 @@ public class HandleFailuresTest {
         Assert.assertEquals(lh.getLedgerMetadata().getAllEnsembles().get(2L), Lists.newArrayList(b5, b2, b4));
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testNoAddsAreCompletedWhileFailureHandlingInProgress() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
@@ -449,7 +445,7 @@ public class HandleFailuresTest {
         Assert.assertEquals(lh.getLedgerMetadata().getAllEnsembles().get(1L), Lists.newArrayList(b1, b2, b4));
     }
 
-    @Test
+    @Test(timeout = 30000)
     public void testHandleFailureBookieNotInWriteSet() throws Exception {
         MockClientContext clientCtx = MockClientContext.create();
         Versioned<LedgerMetadata> md = ClientUtil.setupLedger(clientCtx, 10L,
