@@ -275,6 +275,7 @@ public abstract class BookKeeperClusterTestCase {
             bkc.close();
         }
 
+        stopReplicationService();
         for (ServerTester t : servers) {
             t.shutdown();
         }
@@ -316,6 +317,7 @@ public abstract class BookKeeperClusterTestCase {
     }
 
     protected void stopAllBookies(boolean shutdownClient) throws Exception {
+        stopReplicationService();
         for (ServerTester t : servers) {
             t.shutdown();
         }
@@ -607,6 +609,7 @@ public abstract class BookKeeperClusterTestCase {
             throws Exception {
         // shut down bookie server
         List<ServerConfiguration> confs = new ArrayList<>();
+        stopReplicationService();
         for (ServerTester server : servers) {
             server.shutdown();
             confs.add(server.getConfiguration());
@@ -898,14 +901,18 @@ public abstract class BookKeeperClusterTestCase {
         }
 
         public void startAutoRecovery() throws Exception {
-            LOG.debug("Starting Auditor Recovery for the bookie: {}", address);
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Starting Auditor Recovery for the bookie: {}", address);
+            }
             autoRecovery = new AutoRecoveryMain(conf);
             autoRecovery.start();
         }
 
         public void stopAutoRecovery() {
             if (autoRecovery != null) {
-                LOG.debug("Shutdown Auditor Recovery for the bookie: {}", address);
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Shutdown Auditor Recovery for the bookie: {}", address);
+                }
                 autoRecovery.shutdown();
             }
         }
@@ -959,7 +966,9 @@ public abstract class BookKeeperClusterTestCase {
             }
 
             if (autoRecovery != null) {
-                LOG.debug("Shutdown auto recovery for bookieserver: {}", address);
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Shutdown auto recovery for bookie server: {}", address);
+                }
                 autoRecovery.shutdown();
             }
         }
