@@ -27,14 +27,11 @@ import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 import io.netty.buffer.UnpooledByteBufAllocator;
-
 import java.io.File;
 import java.util.HashSet;
-
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.bookie.MockUncleanShutdownDetection;
 import org.apache.bookkeeper.bookie.TestBookieImpl;
-import org.apache.bookkeeper.common.testing.annotations.FlakyTest;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.proto.BookieServer;
 import org.apache.bookkeeper.stats.NullStatsLogger;
@@ -124,7 +121,7 @@ public class BookieZKExpireTest extends BookKeeperClusterTestCase {
     Attempt to reconnect by BookieStateManager's RegistrationManager listener
     will fail (even if retry it many times).
     */
-    @FlakyTest(value = "https://github.com/apache/bookkeeper/issues/4142")
+    @Test
     @SuppressWarnings("deprecation")
     public void testBookieServerZKSessionExpireBehaviour() throws Exception {
         // 6000 is minimum due to default tick time
